@@ -21,6 +21,7 @@ use crate::{
     RnpContext,
     RnpKey,
     str_to_rnp_buffer,
+    conversions::ToRnpId,
     error::*,
 };
 
@@ -60,6 +61,19 @@ fn rnp_signature_handle_destroy(sig: *mut RnpSignature) -> RnpResult {
     RNP_SUCCESS
 }
 
+#[no_mangle] pub unsafe extern "C"
+fn rnp_signature_get_hash_alg(sig: *const RnpSignature,
+                              hash_alg: *mut *mut c_char)
+                           -> RnpResult {
+    rnp_function!(rnp_signature_get_keyid, crate::TRACE);
+    let sig = assert_ptr_ref!(sig);
+    assert_ptr!(hash_alg);
+
+    *hash_alg = str_to_rnp_buffer(sig.hash_algo().to_rnp_id());
+    RNP_SUCCESS
+}
+
+
 // XXX: This interface is terrible.  It retrieves issuer information
 // from the signature in an unsafe way.
 #[no_mangle] pub unsafe extern "C"
@@ -155,3 +169,27 @@ fn rnp_signature_is_valid(sig: *const RnpSignature,
     // these signatures, don't complicate the implementation... yet.
     RNP_ERROR_SIGNATURE_INVALID
 }
+
+/// Replacement for ComponentAmalgamation::signatures.
+///
+/// This started out as a replacement for a function that was not
+/// available in sequoia-openpgp 1.0.  However, we now mark
+/// self-signatures as valid, a functionality that is not available in
+/// the stock version of the function.
+use openpgp::cert::amalgamation::ComponentAmalgamation;
+pub fn ca_signatures<'a, T>(ca: &ComponentAmalgamation<'a, T>)
+                            -> impl Iterator<Item = (&'a Signature, Option<bool>)>
+{
+    use openpgp::packet::signature::subpacket::SubpacketTag;
+    ca.signatures().map(|sig| {
+        // If Sequoia verified a signature, it will set the
+        // authenticated flag of subpackets to true.  Thus, we know a
+        // signature is "valid" iff the signature creation time has
+        // been authenticated.
+        let valid = sig.subpacket(SubpacketTag::SignatureCreationTime)
+            .map(|sct| if sct.authenticated() { Some(true) } else { None })
+            .unwrap_or(None);
+
+        (sig, valid)
+    })
+}
