@@ -25,6 +25,7 @@ use crate::{
     RnpContext,
     RnpSignature,
     error::*,
+    signature::ca_signatures,
 };
 
 pub struct RnpUserID {
@@ -102,20 +103,6 @@ fn rnp_uid_handle_destroy(uid: *mut RnpUserID)
     RNP_SUCCESS
 }
 
-/// Replacement for ComponentAmalgamation::signatures.
-///
-/// XXX: This is not available in sequoia-openpgp 1.0.  Drop this
-/// workaround once we depend on >= 1.1.
-use openpgp::cert::amalgamation::ComponentAmalgamation;
-use openpgp::packet::Signature;
-fn signatures<'a>(ca: ComponentAmalgamation<'a, UserID>)
-                  -> impl Iterator<Item = (&'a Signature, Option<bool>)> {
-    ca.self_revocations().map(|sig| (sig, Some(true)))
-        .chain(ca.self_signatures().map(|sig| (sig, Some(true))))
-        .chain(ca.certifications().map(|sig| (sig, None)))
-        .chain(ca.other_revocations().map(|sig| (sig, None)))
-}
-
 // XXX: what signatures to consider?  all of them?
 #[no_mangle] pub unsafe extern "C"
 fn rnp_uid_get_signature_at(uid: *const RnpUserID,
@@ -126,8 +113,8 @@ fn rnp_uid_get_signature_at(uid: *const RnpUserID,
     let uid = assert_ptr_ref!(uid);
     let sig = assert_ptr_mut!(sig);
 
-    if let Some((s, valid)) = signatures(
-        uid.cert.userids().nth(uid.nth_uid)
+    if let Some((s, valid)) = ca_signatures(
+        &uid.cert.userids().nth(uid.nth_uid)
             .expect("we know it's there")
     ).nth(idx)
     {
@@ -148,8 +135,8 @@ fn rnp_uid_get_signature_count(uid: *const RnpUserID,
     let uid = assert_ptr_ref!(uid);
     let count = assert_ptr_mut!(count);
 
-    *count = signatures(
-        uid.cert.userids().nth(uid.nth_uid)
+    *count = ca_signatures(
+        &uid.cert.userids().nth(uid.nth_uid)
             .expect("we know it's there")
     ).count();
     RNP_SUCCESS
